@@ -441,7 +441,7 @@ list_leafdir(
 }
 
 /* Read the directory, display contents. */
-int
+static int
 listdir(
 	struct xfs_inode	*dp)
 {
@@ -450,7 +450,7 @@ listdir(
 		.geo		= dp->i_mount->m_dir_geo,
 	};
 	int			error;
-	int			isblock;
+	bool			isblock;
 
 	if (dp->i_df.if_format == XFS_DINODE_FMT_LOCAL)
 		return list_sfdir(&args);
