@@ -39,6 +39,17 @@
 #define trace_xfs_alloc_vextent_loopfailed(a)	((void) 0)
 #define trace_xfs_alloc_vextent_allfailed(a)	((void) 0)
 
+#define trace_xfs_attr_defer_add(...)		((void) 0)
+#define trace_xfs_attr_defer_replace(...)	((void) 0)
+#define trace_xfs_attr_defer_remove(...)	((void) 0)
+#define trace_xfs_attr_sf_addname_return(...)	((void) 0)
+#define trace_xfs_attr_set_iter_return(...)	((void) 0)
+#define trace_xfs_attr_leaf_addname_return(...)	((void) 0)
+#define trace_xfs_attr_node_addname_return(...)	((void) 0)
+#define trace_xfs_attr_remove_iter_return(...)	((void) 0)
+#define trace_xfs_attr_rmtval_alloc(...)	((void) 0)
+#define trace_xfs_attr_rmtval_remove_return(...) ((void) 0)
+
 #define trace_xfs_log_recover_item_add_cont(a,b,c,d)	((void) 0)
 #define trace_xfs_log_recover_item_add(a,b,c,d)	((void) 0)
 
@@ -63,6 +74,8 @@
 #define trace_xfs_iext_insert(a,b,c,d)		((void) 0)
 #define trace_xfs_iext_remove(a,b,c,d)		((void) 0)
 
+#define trace_xfs_defer_relog_intent(a,b)	((void) 0)
+
 #define trace_xfs_dir2_grow_inode(a,b)		((void) 0)
 #define trace_xfs_dir2_shrink_inode(a,b)	((void) 0)
 
@@ -186,6 +199,8 @@
 #define trace_xfs_trans_read_buf_recur(a)	((void) 0)
 #define trace_xfs_trans_read_buf(a)		((void) 0)
 #define trace_xfs_trans_commit(a,b)		((void) 0)
+#define trace_xfs_trans_resv_calc_minlogsize(a,b,c) ((void) 0)
+#define trace_xfs_log_get_max_trans_res(a,b)	((void) 0)
 
 #define trace_xfs_defer_cancel(a,b)		((void) 0)
 #define trace_xfs_defer_pending_commit(a,b)	((void) 0)
