@@ -6,59 +6,62 @@
 /*
  * Simple memory interface
  */
-
-kmem_zone_t *
-kmem_zone_init(int size, char *name)
+struct kmem_cache *
+kmem_cache_create(const char *name, unsigned int size, unsigned int align,
+		unsigned int slab_flags, void (*ctor)(void *))
 {
-	kmem_zone_t	*ptr = malloc(sizeof(kmem_zone_t));
+	struct kmem_cache	*ptr = malloc(sizeof(struct kmem_cache));
 
 	if (ptr == NULL) {
-		fprintf(stderr, _("%s: zone init failed (%s, %d bytes): %s\n"),
-			progname, name, (int)sizeof(kmem_zone_t),
+		fprintf(stderr, _("%s: cache init failed (%s, %d bytes): %s\n"),
+			progname, name, (int)sizeof(struct kmem_cache),
 			strerror(errno));
 		exit(1);
 	}
-	ptr->zone_unitsize = size;
-	ptr->zone_name = name;
+	ptr->cache_unitsize = size;
+	ptr->cache_name = name;
 	ptr->allocated = 0;
+	ptr->align = align;
+	ptr->ctor = ctor;
+
 	return ptr;
 }
 
 int
-kmem_zone_destroy(kmem_zone_t *zone)
+kmem_cache_destroy(struct kmem_cache *cache)
 {
 	int	leaked = 0;
 
-	if (getenv("LIBXFS_LEAK_CHECK") && zone->allocated) {
+	if (getenv("LIBXFS_LEAK_CHECK") && cache->allocated) {
 		leaked = 1;
-		fprintf(stderr, "zone %s freed with %d items allocated\n",
-				zone->zone_name, zone->allocated);
+		fprintf(stderr, "cache %s freed with %d items allocated\n",
+				cache->cache_name, cache->allocated);
 	}
-	free(zone);
+	free(cache);
 	return leaked;
 }
 
 void *
-kmem_cache_alloc(kmem_zone_t *zone, gfp_t flags)
+kmem_cache_alloc(struct kmem_cache *cache, gfp_t flags)
 {
-	void	*ptr = malloc(zone->zone_unitsize);
+	void	*ptr = malloc(cache->cache_unitsize);
 
 	if (ptr == NULL) {
-		fprintf(stderr, _("%s: zone alloc failed (%s, %d bytes): %s\n"),
-			progname, zone->zone_name, zone->zone_unitsize,
+		fprintf(stderr, _("%s: cache alloc failed (%s, %d bytes): %s\n"),
+			progname, cache->cache_name, cache->cache_unitsize,
 			strerror(errno));
 		exit(1);
 	}
-	zone->allocated++;
+	cache->allocated++;
 	return ptr;
 }
 
 void *
-kmem_cache_zalloc(kmem_zone_t *zone, gfp_t flags)
+kmem_cache_zalloc(struct kmem_cache *cache, gfp_t flags)
 {
-	void	*ptr = kmem_cache_alloc(zone, flags);
+	void	*ptr = kmem_cache_alloc(cache, flags);
 
-	memset(ptr, 0, zone->zone_unitsize);
+	memset(ptr, 0, cache->cache_unitsize);
 	return ptr;
 }
 
