@@ -602,6 +602,16 @@ inject_lost_extent(
 	return -libxfs_trans_commit(tp);
 }
 
+void
+check_rtmetadata(
+	struct xfs_mount	*mp)
+{
+	rtinit(mp);
+	generate_rtinfo(mp, btmcompute, sumcompute);
+	check_rtbitmap(mp);
+	check_rtsummary(mp);
+}
+
 void
 phase5(xfs_mount_t *mp)
 {
@@ -671,8 +681,7 @@ phase5(xfs_mount_t *mp)
 	if (mp->m_sb.sb_rblocks)  {
 		do_log(
 		_("        - generate realtime summary info and bitmap...\n"));
-		rtinit(mp);
-		generate_rtinfo(mp, btmcompute, sumcompute);
+		check_rtmetadata(mp);
 	}
 
 	do_log(_("        - reset superblock...\n"));
